/**********************************************************************
 *                                                                    *
 * Copyright (c) 2005 , LCG ROOT MathLib Team                         *
 *                                                                    *
 *                                                                    *
 **********************************************************************/

// Header file for class BasicLorentzRotation
// 
// Created by: Lorenzo Moneta  at Wed May 11 18:22:03 2005
// 
// Last update: Wed May 11 18:22:03 2005
// 
#ifndef ROOT_MATH_BASICLORENTZROTATION
#define ROOT_MATH_BASICLORENTZROTATION 1

#include "GenVector/Rotationfwd.h"
#include "GenVector/Rotation4DMatrix.h"
#include "GenVector/BoostMatrix.h"
#include "GenVector/BasicLorentzVector.h"
// needed for definition of size_t on gcc3.2 
#include <stdlib.h>



namespace ROOT { 

  namespace Math { 



  /** 
     BasicLorentzRotation class for 4D rotations. 
     The class is template on the type representing the 4D rotation which could be a 4D (4x4) matrix or 
     for example in the case of a Boost a symmetric 4D matrix  
  */ 
template<class RotationType >
class BasicLorentzRotation {

 public: 

 
  typedef typename RotationType::Scalar T;
  typedef RotationType RotationCoordinateType;


  /** 
     Default constructor (indentity matrix)
  */ 
  BasicLorentzRotation() {} 

  /**
     Construct from any other Lorentz Rotation (for example a  Lorentz Boost)
  */ 
  template <class ALorentzRotation>
  explicit BasicLorentzRotation( const ALorentzRotation & b) {
  }

  /**
     Construct a Lorentz Rotation from a 3D Rotation 
  */     
  template < class Rotation3DType > 
  explicit BasicLorentzRotation( const Basic3DRotation<Rotation3DType> & r) { 
    // t. b . impl
  }

  /**
     Construct from a Boost  and  a 3D Rotation 
  */     
  template < class Rotation3DType> 
  BasicLorentzRotation( const BasicLorentzRotation<BoostMatrix<T> > & b, const Basic3DRotation<Rotation3DType> & r) { 
    // t. b . impl
  }

  /**
     Construct from a 3D vector representig the boost (beta = vector magnitude) 
   */
  template<class CoordSystem> 
  explicit BasicLorentzRotation(const DisplacementVector3D<CoordSystem> & v) { 
  }

  /**
     Contruct from three numbers representing betaX, betaY, betaZ
   */
  BasicLorentzRotation(const T & betaX, const T & betaY, const T & betaZ) { 
  }

  /**
     Construct from a C array of size at least 16 copying the data  
   */ 
  explicit BasicLorentzRotation( const T * d) : fRotation(d) {}

  /**
     Construct from a linear algebra matrix of size at least 4x4 
  */
  // need to pass size to distringuish from other method
  template<class AMatrix>
  BasicLorentzRotation(const AMatrix & m, size_t size) : fRotation(m,size) {}


  /** 
     Destructor (no operations)
  */ 
  ~BasicLorentzRotation() {} 

  /** 
     Copy constructor
  */ 
  BasicLorentzRotation(const BasicLorentzRotation & r) : 
    fRotation(r.fRotation) {}

  /** 
     Assignment operator
  */ 
  BasicLorentzRotation & operator = (const BasicLorentzRotation & r) { 
    if (this != &r) { 
      fRotation = r.fRotation;
    }
    return *this;
  }

  // accessors

  /** 
      Access to a reference of the object reference representing the rotation (const version). 
      This method couls be used to get then the data (without copying) from the rotation matrix
  */ 
  const RotationType &  type() const {  return fRotation; }

  /** 
      Access to a reference of the object reference representing the rotation (non const version). 
      This method couls be used to get then the data (without copying) from the rotation matrix
  */ 
  RotationType & type() { return fRotation; }



  /**
     access rotation matrix element
  */
  T operator() ( unsigned int i, unsigned int j) const { 
    return fRotation(i,j);
  }

  // get rotation elements
  T xx() const { return fRotation(0,0); }
  T xy() const { return fRotation(0,1); }
  T xz() const { return fRotation(0,2); }
  T xt() const { return fRotation(0,3); }

  // get rotation elements
  T yx() const { return fRotation(1,0); }
  T yy() const { return fRotation(1,1); }
  T yz() const { return fRotation(1,2); }
  T yt() const { return fRotation(1,3); }

  // get rotation elements
  T zx() const { return fRotation(2,0); }
  T zy() const { return fRotation(2,1); }
  T zz() const { return fRotation(2,2); }
  T zt() const { return fRotation(2,3); }

  T tx() const { return fRotation(3,0); }
  T ty() const { return fRotation(3,1); }
  T tz() const { return fRotation(3,2); }
  T tt() const { return fRotation(3,3); }



  /**
     decompose 4x4 rotation matrix in a boost and in a 3D rotation
   */
  template < class Rotation3DType, class ABoost > 
  void decompose( const ABoost & b, const Basic3DRotation<Rotation3DType> & r) { 
    // to be impl.
  }

  /**
     Rotation operation on  a Lorentz Vector 
   */
  template <class ALVector>
  ALVector operator() (const ALVector & v) { 
    return fRotation.rotate(v);
  }

  /**
     Overload operator * for rotation on  a generic vector (has to be Gen3Dvector or LorentzVector)
   */
  template <class ALVector>
  ALVector operator* (const ALVector & v) { 
    return this->operator()(v);
  }

  /**
     Multiply two rotations
   */
  //template< class OtherRotation>
  BasicLorentzRotation & operator *= (const BasicLorentzRotation & r) { 
    // multiply the two matrixis (t.b.imp.)
    return *this;
  }

  /**
     Multiply (combine) two rotations
   */
  //template< class OtherRotation>
  BasicLorentzRotation  operator * (const BasicLorentzRotation & r) { 
    BasicLorentzRotation tmp = *this; 
    tmp*= r;
    // multiply the two matrixis (t.b.im.)
    return tmp;
  }
  
  /**
      invert a rotation
   */
  BasicLorentzRotation & invert() {
    // t. b. impl.
    return *this;
  }

  /**
      return inverse of  a rotation
   */
  // move this as a glibal function ???
  BasicLorentzRotation inverse() {
    // t. b. impl. 
    BasicLorentzRotation tmp;
    return tmp;
  }


protected: 


private: 

  RotationType fRotation; 

}; 

  /** 
      convenient name for Lorentz Rotation
   */ 
  typedef BasicLorentzRotation<Rotation4DMatrix<double> > LorentzRotation;


  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_BASICLORENTZROTATION */
