/**********************************************************************
  *                                                                    *
  * Copyright (c) 2005 moneta,  CERN/PH-SFT                            *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

// Forward declaration for Rotation classes
// 
// Created by: Lorenzo Moneta  at Wed May 11 15:18:21 2005
// 
// Last update: Wed May 11 15:18:21 2005
// 
#ifndef MATHCORE_ROTATIONFWD
#define MATHCORE_ROTATIONFWD 1

#include "GenVector/Vector3Dfwd.h"
// #include "GenVector/Point3Dfwd.h"
// #include "GenVector/LorentzVectorfwd.h"



namespace ROOT { 

  namespace Math { 


  template<class RotationType> class Basic3DRotation; 

  template<typename T> class Rotation3DMatrix;  
  template<typename T> class AxisAngle;  
  template<typename T> class EulerAngles;  

  //  template<typename T> class Rotation;  

  /**
     Rotation based on Euler Angles
  */
  typedef Basic3DRotation<EulerAngles<double > > EulerRotation;  
  //  typedef Basic3DRotation<EulerAngles<float> > EulerRotationF;  

  /**
     Rotation based on axis and angle
   */
  typedef Basic3DRotation<AxisAngle<double> > AxisRotation;  
  //  typedef Basic3DRotation<float, AxisAngle> AxisRotationF;  

  /**
     Standard Rotation based on a 3x3 Matrix
   */
  typedef Basic3DRotation<Rotation3DMatrix<double> > Rotation3D;  
  //  typedef Basic3DRotation<float, Rotation3DMatrix> Rotation3DF;  


  // transfromation
  template<class RotationType, class VectorType > class Basic3DTransformation; 

  typedef Basic3DTransformation< Rotation3DMatrix<double>, Cartesian3D<double> > Transform3D;
  //  typedef Basic3DTransformation< float , Rotation3DMatrix, Cartesian3D> Transform3DF;



  } // end namespace Math

} // end namespace ROOT


#endif /* MATHCORE_ROTATIONFWD */
